@@ -53,11 +53,13 @@ static char rcsid[] __attribute__ ((unused)) ="$Header: /usr/cvsroot/magic-8.0/c
 #include "utils/stack.h"
 
     /* Exports */
-bool CalmaDoLibrary = FALSE;	 /* If TRUE, do not output the top level */
-bool CalmaDoLabels = TRUE;	 /* If FALSE, don't output labels with GDS-II */
-bool CalmaDoLower = TRUE;	 /* If TRUE, allow lowercase labels. */
-bool CalmaFlattenArrays = FALSE; /* If TRUE, output arrays as individual uses */
-bool CalmaAddendum = FALSE;	 /* If TRUE, do not output readonly cell defs */
+bool CalmaDoLibrary = FALSE;	  /* If TRUE, do not output the top level */
+bool CalmaDoLabels = TRUE;	  /* If FALSE, don't output labels with GDS-II */
+bool CalmaDoLower = TRUE;	  /* If TRUE, allow lowercase labels. */
+bool CalmaFlattenArrays = FALSE;  /* If TRUE, output arrays as individual uses */
+bool CalmaAddendum = FALSE;	  /* If TRUE, do not output readonly cell defs */
+bool CalmaNoDateStamp = FALSE;	  /* If TRUE, output zero for creation date stamp */
+bool CalmaAllowUndefined = FALSE; /* If TRUE, allow calls to undefined cells */
 
     /* Experimental stuff---not thoroughly tested (as of Sept. 2007)! */
 bool CalmaContactArrays = FALSE; /* If TRUE, output contacts as subcell arrays */
@@ -304,7 +306,12 @@ CalmaWrite(rootDef, f)
      */
 
     dummy.cu_def = rootDef;
-    DBCellReadArea(&dummy, &rootDef->cd_bbox);
+    if (DBCellReadArea(&dummy, &rootDef->cd_bbox, !CalmaAllowUndefined))
+    {
+	TxError("Failure to read entire subtree of the cell.\n");
+	return FALSE;
+    }
+
     DBFixMismatch();
 
     /*
@@ -396,7 +403,10 @@ calmaDumpStructure(def, outf, calmaDefHash, filename)
 
     /* Output structure begin */
     calmaOutRH(28, CALMA_BGNSTR, CALMA_I2, outf);
-    calmaOutDate(def->cd_timestamp, outf);
+    if (CalmaNoDateStamp)
+    	calmaOutDate(time((time_t *) 0), outf);
+    else
+    	calmaOutDate(def->cd_timestamp, outf);
     calmaOutDate(time((time_t *) 0), outf);
 
     /* Find the structure's unique prefix, in case structure calls subcells */
@@ -603,7 +613,7 @@ calmaFullDump(def, fi, outf, filename)
     FILE *outf;
     char *filename;
 {
-    int version, rval, i;
+    int version, rval;
     char *libname = NULL, uniqlibname[4];
     char *sptr, *viewopts;
     bool isAbstract;
@@ -647,24 +657,29 @@ calmaFullDump(def, fi, outf, filename)
     viewopts = (char *)DBPropGet(def, "LEFview", &isAbstract);
     if ((!isAbstract) || (strcasecmp(viewopts, "no_prefix")))
     {
-
 	/* Generate a SHORT name for this cell (else it is easy to run into the
 	 * GDS 32-character cellname limit).  Save it in the hash record.  The
 	 * chance of generating the same prefix for a library that has items
 	 * with conflicting names is vanishingly small, but to be pedantic, store
 	 * the prefix in a hash table and check to make sure that uniqueness is
-	 * ensured.
+	 * ensured.  NOTE:  The first character of a SPICE name cannot be a
+	 * number.  Therefore the first character is alphabetical, and the second
+	 * is alphanumeric.  There are only 936 possible combinations, but this
+	 * is only meant to distinguish cells in large IP blocks of unknown
+	 * origin, of which only a limited number would be expected.  Beware
+	 * the implications for LVS, as the prefixed names from layout would
+	 * need to be compared to un-prefixed names from another netlist.
 	 */
 	while (TRUE)
 	{
 	    HashEntry *he2;
 
-	    for (i = 0; i < 2; i++) {
-		rval = random() % 62;
-		rval = (rval < 26) ? ('A' + rval) : ((rval < 52) ? ('a' + rval - 26) :
-			('0' + rval - 52));
-		uniqlibname[i] = (char)(rval & 127);
-	    }
+	    rval = random() % 26;
+	    rval = 'A' + rval;
+	    uniqlibname[0] = (char)(rval & 127);
+	    rval = random() % 36;
+	    rval = (rval < 26) ? ('A' + rval) : ('0' + rval - 26);
+	    uniqlibname[1] = (char)(rval & 127);
 	    uniqlibname[2] = '_';
 	    uniqlibname[3] = '\0';
 	    he2 = HashLookOnly(&calmaPrefixHash, uniqlibname);
@@ -799,15 +814,19 @@ calmaProcessDef(def, outf, do_library)
 
     if (isReadOnly && hasContent && CalmaAddendum) return (0);
 
-    /* Give some feedback to the user */
-    TxPrintf("   Writing cell %s\n", def->cd_name);
-
     /*
      * Output the definitions for any of our descendants that have
      * not already been output.  Numbers are assigned to the subcells
-     * as they are output.
+     * as they are output.  If the cell will get a "full dump" (by
+     * having GDS_START but no GDS_END), then do not output any subcells,
+     * as they are expected to be in the referenced GDS file.
      */
-    (void) DBCellEnum(def, calmaProcessUse, (ClientData) outf);
+    if (!hasContent || hasGDSEnd)
+	if (DBCellEnum(def, calmaProcessUse, (ClientData) outf) != 0)
+	    return 1;
+
+    /* Give some feedback to the user */
+    TxPrintf("   Generating output for cell %s\n", def->cd_name);
 
     if (isReadOnly && hasContent)
     {
@@ -826,14 +845,22 @@ calmaProcessDef(def, outf, do_library)
 	    /* This is a rare error, but if the subcell is inside	*/
 	    /* another vendor GDS, it would not normally be output.	*/
 
-	    DBPropGet(def->cd_parents->cu_parent, "GDS_FILE", &isReadOnly);
-	    if (!isReadOnly || isAbstract)
-		TxError("Calma output error:  Can't find GDS file \"%s\" "
-				"for vendor cell \"%s\".  Using magic's "
-				"internal definition\n", filename,
-				def->cd_name);
+	    DBPropGet((def->cd_parents->cu_parent == NULL) ? def :
+			def->cd_parents->cu_parent, "GDS_FILE", &isReadOnly);
 	    if (isReadOnly)
+	    {
 		def->cd_flags |= CDVENDORGDS;
+		return 0;	/* Ignore without raising an error */
+	    }
+
+	    TxError("Calma output error:  Can't find GDS file \"%s\" "
+				"for vendor cell \"%s\".  It will not be output.\n",
+				filename, def->cd_name);
+
+	    if (CalmaAllowUndefined)
+		return 0;
+	    else
+		return 1;
 	}
 	else if (isAbstract || (!hasGDSEnd))
 	{
@@ -867,7 +894,10 @@ calmaProcessDef(def, outf, do_library)
 
 		/* Output structure header */
 		calmaOutRH(28, CALMA_BGNSTR, CALMA_I2, outf);
-		calmaOutDate(def->cd_timestamp, outf);
+    		if (CalmaNoDateStamp)
+		    calmaOutDate(time((time_t *) 0), outf);
+		else
+		    calmaOutDate(def->cd_timestamp, outf);
 		calmaOutDate(time((time_t *) 0), outf);
 
 		/* Name structure the same as the magic cellname */
@@ -922,7 +952,7 @@ calmaProcessDef(def, outf, do_library)
 	if (!do_library)
 	    calmaOutFunc(def, outf, &TiPlaneRect);
 
-    return (0);
+    return 0;
 }
 
 
@@ -964,7 +994,10 @@ calmaOutFunc(def, f, cliprect)
 
     /* Output structure begin */
     calmaOutRH(28, CALMA_BGNSTR, CALMA_I2, f);
-    calmaOutDate(def->cd_timestamp, f);
+    if (CalmaNoDateStamp)
+    	calmaOutDate(time((time_t *) 0), f);
+    else
+	calmaOutDate(def->cd_timestamp, f);
     calmaOutDate(time((time_t *) 0), f);
 
     /* Output structure name */
@@ -2693,7 +2726,10 @@ calmaOutHeader(rootDef, f)
 
     /* Beginning of library */
     calmaOutRH(28, CALMA_BGNLIB, CALMA_I2, f);
-    calmaOutDate(rootDef->cd_timestamp, f);
+    if (CalmaNoDateStamp)
+    	calmaOutDate(time((time_t *) 0), f);
+    else
+    	calmaOutDate(rootDef->cd_timestamp, f);
     calmaOutDate(time((time_t *) 0), f);
 
     /* Library name (name of root cell) */
