@@ -309,6 +309,11 @@ CmdLabel(w, cmd)
     CmdLabelProc(p, font, size, rotate, offx, offy, pos, sticky, type);
 }
 
+#define LOAD_NOWINDOW	  0
+#define LOAD_DEREFERENCE  1
+#define LOAD_FORCE	  2
+#define LOAD_QUIET	  3
+#define LOAD_FAIL	  4
 
 /*
  * ----------------------------------------------------------------------------
@@ -318,7 +323,7 @@ CmdLabel(w, cmd)
  * Implement the "load" command.
  *
  * Usage:
- *	load [name [scaled n [d]]] [-force] [-nowindow] [-dereference]
+ *	load [name [scaled n [d]]] [-force] [-nowindow] [-dereference] [-quiet] [-fail]
  *
  * If name is supplied, then the window containing the point tool is
  * remapped so as to edit the cell with the given name.
@@ -354,29 +359,58 @@ CmdLoad(w, cmd)
 {
     int n = 1;
     int d = 1;
+    int option;
     int locargc = cmd->tx_argc;
     bool ignoreTech = FALSE;
     bool noWindow = FALSE;
     bool dereference = FALSE;
+    bool beQuiet = FALSE;
+    bool failNotFound = FALSE;
+    bool saveVerbose;
+    unsigned char flags;
     int keepGoing();			/* forward declaration */
+    extern bool DBVerbose;		/* from DBio.c */
+
+    saveVerbose = DBVerbose;
+
+    static char *cmdLoadOption[] =
+    {
+	"-nowindow	load file but do not display in the layout window",
+	"-dereference	use search paths and ignore embedded cell paths in file",
+	"-force 	load file even if tech in header does not match",
+	"-quiet		no alert if file does not exist",
+	"-fail		if file does not exist, do not create a new cell",
+	NULL
+    };
+
+    while (cmd->tx_argv[locargc - 1][0] == '-')
+    {
+    	option = Lookup(cmd->tx_argv[locargc - 1], cmdLoadOption);
+	switch (option)
+	{
+	    case LOAD_NOWINDOW:
+	    	noWindow = TRUE;
+		break;
+	    case LOAD_DEREFERENCE:
+	    	dereference = TRUE;
+		break;
+	    case LOAD_FORCE:
+	    	ignoreTech = TRUE;
+		break;
+	    case LOAD_QUIET:
+	    	beQuiet = TRUE;
+		break;
+	    case LOAD_FAIL:
+	    	failNotFound = TRUE;
+		break;
+	    default:
+		TxError("No such option \"%s\".\n", cmd->tx_argv[locargc - 1]);
+	}
+	locargc--;
+    }
 
     if (locargc > 2)
     {
-	if (!strncmp(cmd->tx_argv[locargc - 1], "-nowindow", 8))
-	{
-	    locargc--;
-	    noWindow = TRUE;
-	}
-	if (!strncmp(cmd->tx_argv[locargc - 1], "-deref", 5))
-	{
-	    locargc--;
-	    dereference = TRUE;
-	}
-	if (!strncmp(cmd->tx_argv[locargc - 1], "-force", 6))
-	{
-	    locargc--;
-	    ignoreTech = TRUE;
-	}
 	if ((locargc >= 4) && !strncmp(cmd->tx_argv[2], "scale", 5) &&
 		StrIsInt(cmd->tx_argv[3]))
 	{
@@ -397,7 +431,7 @@ CmdLoad(w, cmd)
 	else if (!ignoreTech && !noWindow && !dereference)
 	{
 	    TxError("Usage: %s name [scaled n [d]] [-force] "
-			    "[-nowindow] [-dereference]\n",
+			    "[-nowindow] [-dereference] [-quiet] [-fail]\n",
 			    cmd->tx_argv[0]);
 	    return;
 	}
@@ -422,8 +456,15 @@ CmdLoad(w, cmd)
 	    *(cmd->tx_argv[1] + strlen(cmd->tx_argv[1]) - 1) = '\0';
 	}
 #endif
-	DBWloadWindow((noWindow == TRUE) ? NULL : w, cmd->tx_argv[1],
-			ignoreTech, FALSE, dereference);
+	DBVerbose = !beQuiet;
+	flags = 0;
+	if (ignoreTech) flags |= DBW_LOAD_IGNORE_TECH;
+	if (dereference) flags |= DBW_LOAD_DEREFERENCE;
+	if (failNotFound) flags |= DBW_LOAD_FAIL;
+	if (beQuiet) flags |= DBW_LOAD_QUIET;
+
+	DBWloadWindow((noWindow == TRUE) ? NULL : w, cmd->tx_argv[1], flags);
+	DBVerbose = saveVerbose;
 
 	if ((n > 1) || (d > 1))
 	{
@@ -456,7 +497,12 @@ CmdLoad(w, cmd)
 	    ReduceFraction(&DBLambda[0], &DBLambda[1]);
 	}
     }
-    else DBWloadWindow(w, (char *) NULL, TRUE, FALSE, FALSE);
+    else
+    {
+	DBVerbose = !beQuiet;
+	DBWloadWindow(w, (char *) NULL, DBW_LOAD_IGNORE_TECH);
+	DBVerbose = saveVerbose;
+    }
 }
 
 /*
@@ -1298,7 +1344,7 @@ CmdPort(w, cmd)
     char **msg;
     int argstart;
     int i, refidx, idx, pos, type, option, argc;
-    unsigned short dirmask;
+    unsigned int dirmask;
     bool found;
     bool nonEdit = FALSE;
     Label *lab, *sl;
