@@ -1818,7 +1818,7 @@ cmdWriteallFunc(def, cmd)
 	    cmdSaveCell(def, (char *) NULL, FALSE, TRUE);
 	    break;
 	case 1:		/* Flush */
-	    cmdFlushCell(def);
+	    cmdFlushCell(def, FALSE);
 	    break;
 	case 2:		/* Skip */
 	    break;
@@ -1885,9 +1885,9 @@ CmdXload(w, cmd)
     {
 	if (CmdIllegalChars(cmd->tx_argv[1], "[],", "Cell name"))
 	    return;
-	DBWloadWindow(w, cmd->tx_argv[1], FALSE, TRUE, FALSE);
+	DBWloadWindow(w, cmd->tx_argv[1], DBW_LOAD_EXPAND);
     }
-    else DBWloadWindow(w, (char *) NULL, FALSE, TRUE, FALSE);
+    else DBWloadWindow(w, (char *) NULL, DBW_LOAD_EXPAND);
 }
 
 /*
@@ -1926,7 +1926,7 @@ CmdXor(w, cmd)
 
     PaintResultType	DBXORResultTbl[NP][NT][NT];
     PaintResultType	(*curPaintSave)[NT][NT];
-    void		(*curPlaneSave)();
+    int			(*curPlaneSave)();
 
     int p, t, h;
 
