@@ -3,21 +3,21 @@
  *
  * Cell copying (yank and stuff)
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBcellcopy.c,v 1.11 2010/05/11 19:28:53 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBcellcopy.c,v 1.13 2010/06/24 12:37:15 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -48,7 +48,7 @@ static PaintResultType (*dbCurPaintTbl)[NT][NT] = DBPaintResultTbl;
  * such as the design-rule checker that need to use, for example,
  * DBPaintPlaneMark instead of the standard version.
  */
-static void (*dbCurPaintPlane)() = DBPaintPlaneWrapper;
+static int (*dbCurPaintPlane)() = DBPaintPlaneWrapper;
 
     /* Structure passed to DBTreeSrTiles() */
 struct copyAllArg
@@ -56,6 +56,7 @@ struct copyAllArg
     TileTypeBitMask	*caa_mask;	/* Mask of tile types to be copied */
     Rect		 caa_rect;	/* Clipping rect in target coords */
     CellUse		*caa_targetUse;	/* Use to which tiles are copied */
+    void		(*caa_func)();	/* Callback function for off-grid points */
     Rect		*caa_bbox;	/* Bbox of material copied (in
 					 * targetUse coords).  Used only when
 					 * copying cells.
@@ -96,7 +97,7 @@ struct copyLabelArg
  * ----------------------------------------------------------------------------
  */
 
-void
+int
 DBPaintPlaneWrapper(def, pNum, type, area, undo)
     CellDef *def;
     int pNum;
@@ -104,23 +105,16 @@ DBPaintPlaneWrapper(def, pNum, type, area, undo)
     Rect *area;
     PaintUndoInfo *undo;
 {
-#ifdef NONMANHATTAN
     TileType loctype = type & TT_LEFTMASK;
     Rect expand;
-#else
-    TileType loctype = type;
-#endif
+    int result;
 
     undo->pu_pNum = pNum;
-#ifdef NONMANHATTAN
-    DBNMPaintPlane(def->cd_planes[pNum], type, area,
+    result = DBNMPaintPlane(def->cd_planes[pNum], type, area,
 		dbCurPaintTbl[pNum][loctype], undo);
     GEO_EXPAND(area, 1, &expand);
-    DBMergeNMTiles(def->cd_planes[pNum], &expand);
-#else
-    DBPaintPlane(def->cd_planes[pNum], area,
-		dbCurPaintTbl[pNum][loctype], undo);
-#endif
+    DBMergeNMTiles(def->cd_planes[pNum], &expand, undo);
+    return result;
 }
 
 /*
@@ -135,7 +129,7 @@ DBPaintPlaneWrapper(def, pNum, type, area, undo)
  * ----------------------------------------------------------------------------
  */
 
-void
+int
 DBPaintPlaneMark(def, pNum, type, area, undo)
     CellDef *def;
     int pNum;
@@ -143,20 +137,32 @@ DBPaintPlaneMark(def, pNum, type, area, undo)
     Rect *area;
     PaintUndoInfo *undo;
 {
-#ifdef NONMANHATTAN
     TileType loctype = type & TT_LEFTMASK;
-#else
-    TileType loctype = type;
-#endif
 
     undo->pu_pNum = pNum;
-#ifdef NONMANHATTAN
-    DBNMPaintPlane0(def->cd_planes[pNum], type, area,
-		dbCurPaintTbl[pNum][loctype], undo, TRUE);
-#else
-    DBPaintPlane0(def->cd_planes[pNum], area,
-		dbCurPaintTbl[pNum][loctype], undo, TRUE);
-#endif
+    return DBNMPaintPlane0(def->cd_planes[pNum], type, area,
+		dbCurPaintTbl[pNum][loctype], undo, (unsigned char)PAINT_MARK);
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+DBPaintPlaneXor(def, pNum, type, area, undo)
+    CellDef *def;
+    int pNum;
+    TileType type;
+    Rect *area;
+    PaintUndoInfo *undo;
+{
+    TileType loctype = type & TT_LEFTMASK;
+
+    undo->pu_pNum = pNum;
+    return DBNMPaintPlane0(def->cd_planes[pNum], type, area,
+		dbCurPaintTbl[pNum][loctype], undo, (unsigned char)PAINT_XOR);
 }
 
 /*
@@ -166,13 +172,13 @@ DBPaintPlaneMark(def, pNum, type, area, undo)
  *
  *    This function calls DBPaintPlane, but first checks if the type
  *    being painted is an active layer.  If the type is a contact,
- *    then the residues are checked to see if they are active layers. 
+ *    then the residues are checked to see if they are active layers.
  *    Painting proceeds accordingly.
  *
  * ----------------------------------------------------------------------------
  */
 
-void
+int
 DBPaintPlaneActive(def, pNum, type, area, undo)
     CellDef *def;
     int pNum;
@@ -180,11 +186,7 @@ DBPaintPlaneActive(def, pNum, type, area, undo)
     Rect *area;
     PaintUndoInfo *undo;
 {
-#ifdef NONMANHATTAN
     TileType loctype = type & TT_LEFTMASK;
-#else
-    TileType loctype = type;
-#endif
     TileType t;
 
     if (DBIsContact(loctype))
@@ -201,14 +203,15 @@ DBPaintPlaneActive(def, pNum, type, area, undo)
 			DBPaintPlaneWrapper(def, pNum, t | (type &
 				(TT_SIDE | TT_DIRECTION | TT_DIAGONAL)),
 				area, undo);
-	    return;
+	    return 0;
 	}
     }
     if (TTMaskHasType(&DBActiveLayerBits, loctype))
-	DBPaintPlaneWrapper(def, pNum, type, area, undo);
+	return DBPaintPlaneWrapper(def, pNum, type, area, undo);
+    else
+	return 0;
 }
 
-#ifdef NONMANHATTAN
 /*
  *-----------------------------------------------------------------------------
  *
@@ -243,13 +246,13 @@ DBCellCopyManhattanPaint(scx, mask, xMask, targetUse)
 
     arg.caa_mask = mask;
     arg.caa_targetUse = targetUse;
+    arg.caa_func = NULL;
     GEOTRANSRECT(&scx->scx_trans, &scx->scx_area, &arg.caa_rect);
 
     (void) DBTreeSrTiles(scx, mask, xMask, dbCopyManhattanPaint, (ClientData) &arg);
 }
-#endif
 
-
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -284,6 +287,7 @@ DBCellCopyAllPaint(scx, mask, xMask, targetUse)
 
     arg.caa_mask = mask;
     arg.caa_targetUse = targetUse;
+    arg.caa_func = NULL;
     GEOTRANSRECT(&scx->scx_trans, &scx->scx_area, &arg.caa_rect);
 
     /* Add any stacking types for the search (but not to mask passed as arg!) */
@@ -292,7 +296,52 @@ DBCellCopyAllPaint(scx, mask, xMask, targetUse)
 
     DBTreeSrTiles(scx, &locMask, xMask, dbCopyAllPaint, (ClientData) &arg);
 }
-
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * DBCellCheckCopyAllPaint --
+ *
+ * Copy paint from the tree rooted at scx->scx_use to the paint planes
+ * of targetUse, transforming according to the transform in scx.
+ * Only the types specified by typeMask are copied.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Updates the paint planes in targetUse.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void
+DBCellCheckCopyAllPaint(scx, mask, xMask, targetUse, func)
+    SearchContext *scx;		/* Describes root cell to search, area to
+				 * copy, transform from root cell to coords
+				 * of targetUse.
+				 */
+    TileTypeBitMask *mask;	/* Types of tiles to be yanked/stuffed */
+    int xMask;			/* Expansion state mask to be used in search */
+    CellUse *targetUse;		/* Cell into which material is to be stuffed */
+    void (*func)();		/* Function to call on tile split error */
+{
+    TileTypeBitMask locMask;
+    struct copyAllArg arg;
+    int dbCopyAllPaint();
+
+    arg.caa_mask = mask;
+    arg.caa_targetUse = targetUse;
+    arg.caa_func = func;
+    GEOTRANSRECT(&scx->scx_trans, &scx->scx_area, &arg.caa_rect);
+
+    /* Add any stacking types for the search (but not to mask passed as arg!) */
+    locMask = *mask;
+    DBMaskAddStacking(&locMask);
+
+    DBTreeSrTiles(scx, &locMask, xMask, dbCopyAllPaint, (ClientData) &arg);
+}
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -335,7 +384,7 @@ DBCellCopyAllLabels(scx, mask, xMask, targetUse, pArea)
     /* DBTeeSrLabels finds all the labels that we want plus some more.
      * We'll filter out the ones that we don't need.
      */
-    
+
     arg.cla_targetUse = targetUse;
     arg.cla_bbox = pArea;
     if (pArea != NULL)
@@ -390,7 +439,7 @@ dbCopyAllLabels(scx, lab, tpath, arg)
     }
     return 0;
 }
-
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -444,13 +493,13 @@ DBCellCopyPaint(scx, mask, xMask, targetUse)
     for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
 	if (PlaneMaskHasPlane(planeMask, pNum))
 	{
-	    cxp.tc_plane = pNum; /* not used? */
+	    cxp.tc_plane = pNum;
 	    (void) DBSrPaintArea((Tile *) NULL,
 		scx->scx_use->cu_def->cd_planes[pNum], &scx->scx_area,
 		mask, dbCopyAllPaint, (ClientData) &cxp);
 	}
 }
-
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -459,7 +508,7 @@ DBCellCopyPaint(scx, mask, xMask, targetUse)
  * Copy labels from scx->scx_use to targetUse, transforming according to
  * the transform in scx.  Only labels attached to layers of the types
  * specified by mask are copied.  If mask contains the L_LABEL bit, then
- * all labels are copied regardless of their layer.  The area copied is 
+ * all labels are copied regardless of their layer.  The area copied is
  * determined by GEO_LABEL_IN_AREA.
  *
  * Results:
@@ -529,7 +578,6 @@ DBCellCopyLabels(scx, mask, xMask, targetUse, pArea)
 	}
 }
 
-#ifdef NONMANHATTAN
 /***
  *** Filter function for paint: Ignores diagonal (split) tiles for
  *** purposes of selection searches.
@@ -576,9 +624,8 @@ dbCopyManhattanPaint(tile, cxp)
     (*dbCurPaintPlane)(def, pNum, type, &targetRect, &ui);
     return (0);
 }
-#endif
 
-
+
 /***
  *** Filter function for paint
  ***/
@@ -595,6 +642,7 @@ dbCopyAllPaint(tile, cxp)
     CellDef *def;
     TileType type = TiGetTypeExact(tile);
     int pNum = cxp->tc_plane;
+    int result;
     TileTypeBitMask *typeMask;
 
     /*
@@ -604,10 +652,9 @@ dbCopyAllPaint(tile, cxp)
      * a sanity check.
      */
 
-#ifdef NONMANHATTAN
     bool splittile = FALSE;
     TileType dinfo = 0;
-    
+
     if (IsSplit(tile))
     {
 	splittile = TRUE;
@@ -615,7 +662,6 @@ dbCopyAllPaint(tile, cxp)
 	type = (SplitSide(tile)) ? SplitRightType(tile) :
 			SplitLeftType(tile);
     }
-#endif
 
     if (type == TT_SPACE)
 	return 0;
@@ -664,7 +710,6 @@ dbCopyAllPaint(tile, cxp)
 
     def->cd_flags |= CDMODIFIED|CDGETNEWSTAMP;
 
-#ifdef NONMANHATTAN
     /* Nonmanhattan geometry requires slightly different handling. */
     /* Paint the whole tile and clip by erasing areas outside the  */
     /* clipping rectangle.					   */
@@ -675,7 +720,7 @@ dbCopyAllPaint(tile, cxp)
 	int np, i, j;
 
 	GrClipTriangle(&targetRect, &arg->caa_rect, TRUE, dinfo, points, &np);
-	
+
 	if (np == 0)
 	   return(0);
 
@@ -755,18 +800,21 @@ topbottom:
 	}
     }
     else
-#endif
-
-    /* Clip against the target area */
-    GEOCLIP(&targetRect, &arg->caa_rect);
+	/* Clip against the target area */
+	GEOCLIP(&targetRect, &arg->caa_rect);
 
 splitdone:
 
-    (*dbCurPaintPlane)(def, pNum, dinfo | type, &targetRect, &ui);
+    result = (*dbCurPaintPlane)(def, pNum, dinfo | type, &targetRect, &ui);
+    if ((result != 0) && (arg->caa_func != NULL))
+    {
+	/* result == 1 used exclusively for DRC off-grid error flagging */
+	DRCOffGridError(&targetRect);
+    }
 
     return (0);
 }
-
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -826,7 +874,7 @@ DBCellCopyAllCells(scx, xMask, targetUse, pArea)
 
     (void) DBTreeSrCells(scx, xMask, dbCellCopyCellsFunc, (ClientData) &arg);
 }
-
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -876,7 +924,7 @@ DBCellCopyCells(scx, targetUse, pArea)
 
     (void) DBCellSrArea(scx, dbCellCopyCellsFunc, (ClientData) &arg);
 }
-
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -973,7 +1021,7 @@ dbCellCopyCellsFunc(scx, arg)
     }
     return 2;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -988,7 +1036,9 @@ dbCellCopyCellsFunc(scx, arg)
  *	value with another call to this procedure.
  *
  * Side effects:
- *	A new paint table takes effect.
+ *	A new paint table takes effect.  However, if newTable is NULL,
+ *	then the old paint table remains active.  This allows one to
+ *	get a pointer to the active paint table without altering it.
  *
  * ----------------------------------------------------------------------------
  */
@@ -998,10 +1048,10 @@ DBNewPaintTable(newTable))[NT][NT]
     PaintResultType (*newTable)[NT][NT];  /* Address of new paint table. */
 {
     PaintResultType (*oldTable)[NT][NT] = dbCurPaintTbl;
-    dbCurPaintTbl = newTable;
+    if (newTable) dbCurPaintTbl = newTable;
     return oldTable;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1021,11 +1071,11 @@ DBNewPaintTable(newTable))[NT][NT]
  * ----------------------------------------------------------------------------
  */
 
-VoidProc
+IntProc
 DBNewPaintPlane(newProc)
-    void (*newProc)();		/* Address of new procedure */
+    int (*newProc)();		/* Address of new procedure */
 {
-    void (*oldProc)() = dbCurPaintPlane;
+    int (*oldProc)() = dbCurPaintPlane;
     dbCurPaintPlane = newProc;
     return (oldProc);
 }
