@@ -6,16 +6,16 @@
  * The basic outer loop is a non-recursive area enumeration, and
  * the inner loop attempts to avoid merging as much as possible.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -144,7 +144,7 @@ int dbPaintDebug = 0;
 	} \
     }
 
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -215,7 +215,7 @@ dbJoinUndo(tile, splitx, undo)
  * 'undo' can be NULL.
  *
  * Results:
- *	None.
+ *	Always return 0.
  *
  * Side effects:
  *	Modifies the database plane that contains the given tile.
@@ -236,7 +236,7 @@ dbJoinUndo(tile, splitx, undo)
  * ----------------------------------------------------------------------------
  */
 
-void
+int
 DBPaintPlane0(plane, area, resultTbl, undo, method)
     Plane *plane;		/* Plane whose paint is to be modified */
     Rect *area;			/* Area to be changed */
@@ -262,7 +262,7 @@ DBPaintPlane0(plane, area, resultTbl, undo, method)
     bool haschanged;
 
     if (area->r_xtop <= area->r_xbot || area->r_ytop <= area->r_ybot)
-	return;
+	return 0;
 
     /*
      * The following is a modified version of the area enumeration
@@ -638,12 +638,22 @@ done:
     if (method == (unsigned char)PAINT_MARK)
     {
 	/* Now unmark the processed tiles with the same search algorithm */
+	/* Expand the area by one to catch tiles that were clipped at	 */
+	/* the area boundary.						 */
+
+	area->r_xbot -= 1;
+	area->r_ybot -= 1;
+	area->r_xtop += 1;
+	area->r_ytop += 1;
+	start.p_x = area->r_xbot;
+	start.p_y = area->r_ytop - 1;
+
 	tile = plane->pl_hint;
 	GOTOPOINT(tile, &start);
 
-enum2:
 	while (TOP(tile) > area->r_ybot)
 	{
+enum2:
 	    clipTop = TOP(tile);
 	    if (clipTop > area->r_ytop) clipTop = area->r_ytop;
 
@@ -687,6 +697,7 @@ enum2:
 
 done2:
     plane->pl_hint = tile;
+    return 0;
 }
 
 /*
@@ -1053,7 +1064,7 @@ nmenum:
 		while (BOTTOM(tile) > TOP(tp))
 		    tp = RT(tp);
 		if ((RIGHT(tp) != LEFT(tile)) || (BOTTOM(tile) != TOP(tp)))
-		    break;	
+		    break;
 	    }
 	    else
 	    {
@@ -1190,7 +1201,7 @@ nmenum:
 
 		/* Announce merge to undo system */
 		if (undo && UndoIsEnabled())
-		    dbJoinUndo(tile, RIGHT(tile), undo); 
+		    dbJoinUndo(tile, RIGHT(tile), undo);
 
 		/* All's clear to merge.  Again, walk down from tile */
 		tp2 = LB(tile);
@@ -1253,7 +1264,7 @@ nmenum:
 		    tp = tp2;
 		    tp2 = RT(tp2);
 		}
-	 	/* Merge tp and tile */	
+	 	/* Merge tp and tile */
 		TiJoinX(tile, tp, plane);
 		TiSetBody(tile, ttype);
 	    }
@@ -1311,7 +1322,7 @@ nmdone:
  *	return -1.
  *
  *	This routine is called only from DBNMPaintPlane().
- * 
+ *
  * Results:
  *	Returns the new type to be painted, or -1 if the result can
  *	only be generated by quartering the original tile.
@@ -1387,7 +1398,7 @@ DBDiagonalProc(oldtype, cdata)
 	    newtype = new_n;	/* Turned back into a rectangle */
 	    return newtype;
 	}
-	else 
+	else
 	{
 	    newtype = new_w | (new_e << 14);
 	    newtype |= (TT_DIAGONAL | TT_DIRECTION);
@@ -1406,8 +1417,8 @@ DBDiagonalProc(oldtype, cdata)
 	newtype |= TT_SIDE;
 
     return newtype;
-}	
-    
+}
+
 typedef struct
 {
     Rect	*area;		/* An area to be painted with a triangle */
@@ -1432,7 +1443,7 @@ typedef struct
  * paint quadrangular (clipped triangle) areas.
  *
  * Results:
- *	None.
+ *	0 on success, 1 on error splitting a non-manhattan tile
  *
  * Side Effects:
  *	Plane is painted with a diagonal.  The plane may be hacked up
@@ -1442,7 +1453,7 @@ typedef struct
  * ----------------------------------------------------------------------------
  */
 
-void
+int
 DBNMPaintPlane0(plane, exacttype, area, resultTbl, undo, method)
     Plane *plane;		/* Plane whose paint is to be modified */
     TileType exacttype;		/* diagonal info for tile to be changed */
@@ -1468,6 +1479,7 @@ DBNMPaintPlane0(plane, exacttype, area, resultTbl, undo, method)
     int xc, yc, width, height;
     dlong xref, yref;		/* xref, yref can easily exceed 32 bits */
     int resstate;
+    int result = 0;
 
     if (exacttype & TT_DIAGONAL)
     {
@@ -1522,7 +1534,7 @@ DBNMPaintPlane0(plane, exacttype, area, resultTbl, undo, method)
 		    if (resultTbl[oldType] == oldType)
 		    {
 			freeMagic((char *) lr);
-			return;
+			return 0;
 		    }
 		}
 
@@ -1534,7 +1546,7 @@ DBNMPaintPlane0(plane, exacttype, area, resultTbl, undo, method)
 		if (newType == oldType)
 		{
 		    freeMagic((char *) lr);
-	            return;
+	            return 0;
 		}
 
 		/* Watch for the worst-case scenario of attempting to	*/
@@ -1550,9 +1562,9 @@ DBNMPaintPlane0(plane, exacttype, area, resultTbl, undo, method)
 		{
 		    if ((width == 1) || (height == 1))
 		    {
-			DBPaintPlane(plane, &(lr->r_r), resultTbl, undo);
+			result = DBPaintPlane(plane, &(lr->r_r), resultTbl, undo);
 			freeMagic((char *) lr);
-			return;
+			return 1;	/* Flag the error by returning 1 */
 		    }
 
 		    /* lr->r_r is drawn & quartered */
@@ -1595,7 +1607,7 @@ DBNMPaintPlane0(plane, exacttype, area, resultTbl, undo, method)
 
 		    if (newType & TT_DIAGONAL)
 		    {
-			DBPaintPlane(plane, &(lr->r_r), DBSpecialPaintTbl,
+			result = DBPaintPlane(plane, &(lr->r_r), DBSpecialPaintTbl,
 				(PaintUndoInfo *)NULL);
 			tile = plane->pl_hint;
 			GOTOPOINT(tile, &(lr->r_r.r_ll));
@@ -1610,14 +1622,14 @@ DBNMPaintPlane0(plane, exacttype, area, resultTbl, undo, method)
 		    {
 			PaintResultType tempTbl;
 			tempTbl = newType;
-			DBPaintPlane0(plane, &(lr->r_r), &tempTbl, undo, method);
+			result = DBPaintPlane0(plane, &(lr->r_r), &tempTbl,
+				undo, method);
 		    }
 		    else
-			DBPaintPlane(plane, &(lr->r_r), resultTbl, undo);
+			result = DBPaintPlane(plane, &(lr->r_r), resultTbl, undo);
 
 		    freeMagic((char *) lr);
-		    /* goto nmmerge; */
-	            return;
+	            return result;
 		}
 	    }
 
@@ -1745,11 +1757,12 @@ paintrect:
 	    if (resstate == RES_DIAG)
 	    {
 		/* Recursive call to self on sub-area */
-		DBNMPaintPlane0(plane, exacttype, &(lr->r_r), resultTbl, undo, method);
+		result |= DBNMPaintPlane0(plane, exacttype, &(lr->r_r), resultTbl,
+			undo, method);
 	    }
 	    else if ((resstate == RES_LEFT && !dinfo.side) ||
 		     (resstate == RES_RIGHT && dinfo.side)) {
-		DBPaintPlane(plane, &(lr->r_r), resultTbl, undo);
+		result |= DBPaintPlane(plane, &(lr->r_r), resultTbl, undo);
 	    }
 	    /* else: Rectangle does not contain type and should be ignored. */
 nextrect:
@@ -1764,8 +1777,10 @@ nextrect:
 	}
     }
     else
-	DBPaintPlane0(plane, area, resultTbl, undo, (method == PAINT_MARK) ?
+	result = DBPaintPlane0(plane, area, resultTbl, undo, (method == PAINT_MARK) ?
 		method : PAINT_NORMAL);
+
+    return result;
 }
 
 /*
@@ -1825,11 +1840,11 @@ dbMarkClient(tile, clip)
 	tile->ti_client = (ClientData)CLIENTDEFAULT;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
- * dbPaintMerge -- 
+ * dbPaintMerge --
  *
  * The tile 'tp' is to be changed to type 'newtype'.  To maintain
  * maximal horizontal strips, it may be necessary to merge the new
@@ -2020,7 +2035,7 @@ dbPaintMerge(tile, newType, area, plane, mergeFlags, undo, mark)
 
     return (tile);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -2321,11 +2336,11 @@ paintdone:
 done:
     plane->pl_hint = tile;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
- * dbMergeType -- 
+ * dbMergeType --
  *
  * The tile 'tp' is to be changed to type 'newtype'.  To maintain
  * maximal horizontal strips, it may be necessary to merge the new
@@ -2510,7 +2525,7 @@ dbMergeType(tile, newType, plane, mergeFlags, undo, client)
 
     return (tile);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -2537,7 +2552,7 @@ dbMergeType(tile, newType, plane, mergeFlags, undo, client)
  * ----------------------------------------------------------------------------
  */
 
-void
+int
 DBPaintPlaneVert(plane, area, resultTbl, undo)
     Plane *plane;		/* Plane whose paint is to be modified */
     Rect *area;	/* Area to be changed */
@@ -2558,7 +2573,7 @@ DBPaintPlaneVert(plane, area, resultTbl, undo)
     Tile *newtile, *tp;	/* Used for paint */
 
     if (area->r_xtop <= area->r_xbot || area->r_ytop <= area->r_ybot)
-	return;
+	return 0;
 
     /*
      * The following is a modified version of the area enumeration
@@ -2783,12 +2798,13 @@ paintdone:
 
 done:
     plane->pl_hint = tile;
+    return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
- * dbPaintMergeVert -- 
+ * dbPaintMergeVert --
  *
  * The tile 'tp' is to be changed to type 'newtype'.  To maintain
  * maximal vertical strips, it may be necessary to merge the new
@@ -2969,12 +2985,12 @@ dbPaintMergeVert(tile, newType, plane, mergeFlags, undo)
 
     return (tile);
 }
-
+
 #ifdef	PAINTDEBUG
 /*
  * ----------------------------------------------------------------------------
  *
- * dbPaintShowTile -- 
+ * dbPaintShowTile --
  *
  * Show the tile 'tp' in the cell undo->pu_def in a highlighted style,
  * then print a message, wait for more, and erase the highlights.
@@ -3054,7 +3070,7 @@ TiNMSplitY(oldtile, newtile, y, dir, undo)
     Tile *newxtop, *newxbot;
     Rect r;
 
-    height = TOP(*oldtile) - BOTTOM(*oldtile); 
+    height = TOP(*oldtile) - BOTTOM(*oldtile);
     tmpdx = (long)(y - BOTTOM(*oldtile)) * (long)(RIGHT(*oldtile) - LEFT(*oldtile));
     haschanged = (x = (tmpdx % (long)height) << 1) ? ((undo) ? TRUE : FALSE) : FALSE;
     x = (x >= height) ? 1 : 0;
