@@ -47,9 +47,9 @@ extern void CmdEdit(), CmdElement(), CmdErase(), CmdExpand(), CmdExtract();
 extern void CmdFeedback(), CmdFill(), CmdFindBox(), CmdFindLabel(), CmdFlush();
 extern void CmdGetcell(), CmdGrid(), CmdIdentify();
 extern void CmdLabel(), CmdLoad();
-extern void CmdMove(), CmdNetlist(), CmdPaint(), CmdPath(), CmdPlow();
-extern void CmdPolygon(), CmdPort(), CmdProperty();
-extern void CmdSave(), CmdScaleGrid(), CmdSee();
+extern void CmdMove(), CmdNetlist(), CmdOrient(), CmdPaint(), CmdPath();
+extern void CmdPlow(), CmdPolygon(), CmdPort(), CmdProperty();
+extern void CmdRandom(), CmdSave(), CmdScaleGrid(), CmdSee();
 extern void CmdSelect(), CmdSetLabel(), CmdSideways();
 extern void CmdShell(), CmdSnap();
 extern void CmdStretch(), CmdStraighten();
@@ -378,6 +378,9 @@ DBWInitCommands()
 			for information on options",
 	CmdNetlist, FALSE);
 
+    WindAddCommand(DBWclientID,
+	"orient [orientation]		orient selection and box",
+	CmdOrient, FALSE);
     WindAddCommand(DBWclientID,
 	"paint layers|cursor		paint mask information",
 	CmdPaint, FALSE);
@@ -404,6 +407,9 @@ DBWInitCommands()
     WindAddCommand(DBWclientID,
 	"rotate [+/-][deg]	rotate selection and box (counter)clockwise",
 	CmdClockwise, FALSE);		/* "rotate" is alias for "clockwise" */
+    WindAddCommand(DBWclientID,
+	"random [seed [value]]		generate random number or set random seed",
+	CmdRandom, FALSE);
     WindAddCommand(DBWclientID,
 	"save [filename]		save edit cell on disk",
 	CmdSave, FALSE);
