@@ -61,7 +61,35 @@ static DRCCookie drcSubcellCookie = {
     (DRCCookie *) NULL
 };
 
+/* The cookie below is dummied up to provide an error message for
+ * errors that are in a subcell non-interaction area and have been
+ * copied up into the parent without flattening
+ */
+
+static DRCCookie drcInSubCookie = {
+    0, 0, 0, 0,
+    { 0 }, { 0 },
+    0, 0, 0,
+    DRC_IN_SUBCELL_TAG,
+    (DRCCookie *) NULL
+};
+
+/* The following DRC cookie is used when flattening non-Manhattan
+ * shapes results in a non-integer coordinate.  Because the non-
+ * integer coordinate cannot be represented in magic, the position
+ * is flagged as a DRC error.
+ */
+
+static DRCCookie drcOffGridCookie = {
+    0, 0, 0, 0,
+    { 0 }, { 0 },
+    0, 0, 0,
+    DRC_OFFGRID_TAG,
+    (DRCCookie *) NULL
+};
+
 extern int DRCErrorType;
+extern CellDef *DRCErrorDef;
 
 /*
  * ----------------------------------------------------------------------------
@@ -94,6 +122,79 @@ drcFindOtherCells(use, area)
     return 0;
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * drcSubCopyErrors ---
+ *
+ *  For each tile found in drcCopyErrorsFunc(), translate the tile position
+ *  into the coordinate system of the parent cell (represented by the drcTemp
+ *  plane in ClientData) and apply the function passed in the filter, which is
+ *  whatever function handles DRC errors inside an error tile (which is
+ *  different for "drc why" commands than for "drc check".
+ *
+ *  Returns:
+ *	0 to keep the search going.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+drcSubCopyErrors(tile, cxp)
+    Tile *tile;
+    TreeContext *cxp;
+{
+    Rect area;
+    Rect destArea;
+    struct drcClientData *arg = (struct drcClientData *)cxp->tc_filter->tf_arg;
+
+    TiToRect(tile, &area);
+    GeoClip(&area, &cxp->tc_scx->scx_area);
+    GeoTransRect(&cxp->tc_scx->scx_trans, &area, &destArea);
+
+    (*(arg->dCD_function))(arg->dCD_celldef, &destArea, arg->dCD_cptr,
+		arg->dCD_clientData);
+    (*(arg->dCD_errors))++;
+    
+    return 0;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * drcSubCopyFunc ---
+ *
+ *  This routine is applied for each subcell of a parent def.  It calls
+ *  DBNoTreeSrTiles() with the above routine drcSubCopyErrors(), which
+ *  copies all TT_ERROR_P tiles from the child cell up into the parent.
+ *  This is used only within areas found to be non-interacting with the
+ *  parent, such that any error found in the child cell is guaranteed to
+ *  be a real error, and not one that might be resolved by additional
+ *  material found in the parent or a sibling cell.
+ *
+ *  Added 11/10/2020:  Moved drcArrayFunc() here; this limits the array
+ *  checks to non-interaction areas in the parent cell.
+ *
+ *  Returns:
+ *	Whatever DBNoTreeSrTiles() returns.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+drcSubCopyFunc(scx, cdarg)
+    SearchContext *scx;
+    ClientData cdarg;
+{
+    TileTypeBitMask drcMask;
+
+    /* Create a mask with only TT_ERROR_P in it */
+    TTMaskZero(&drcMask);
+    TTMaskSetType(&drcMask, TT_ERROR_P);
+
+    /* Use DBNoTreeSrTiles() because we want to search only one level down */
+    return DBNoTreeSrTiles(scx, &drcMask, 0, drcSubCopyErrors, cdarg);
+}
 
 /*
  * ----------------------------------------------------------------------------
@@ -146,12 +247,24 @@ drcSubcellFunc(subUse, propagate)
     /* all cells are checked and errors propagate to the top level.	*/
 
     subIntArea = GeoNullRect;
+
+#if (0)
+    /* NOTE:  DRC errors inside a subcell should be ignored for	*/
+    /* the purpose of finding interactions.  Errors should only	*/
+    /* be copied up into the parent when in a non-interaction	*/
+    /* area.  This is done below in DRCFindInteractions().	*/
+    /* (Method added by Tim, 10/15/2020)			*/
+
+    /* Maybe S and PS errors should be pulled here? */
+
     DBSrPaintArea((Tile *) NULL, subUse->cu_def->cd_planes[PL_DRC_ERROR],
 		&TiPlaneRect, &DBAllButSpaceBits, drcIncludeArea,
 		(ClientData) &subIntArea);
 
     GeoTransRect(&(subUse->cu_transform), &subIntArea, &locIntArea);
     GeoInclude(&locIntArea, &intArea);
+#endif
+
     if (!GEO_RECTNULL(&subIntArea)) *propagate = TRUE;
 
     drcCurSub = subUse;
@@ -506,6 +619,38 @@ drcExactOverlapTile(tile, cxp)
     return 0;
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DRCOffGridError ---
+ *
+ * Function to call when a call to DBCellCheckCopyAllPaint() flags an error
+ * indicating that a subcell overlap of non-Manhattan geometry resolves to
+ * an off-grid intersection point.  Note that this is different from the
+ * DRC off-grid check, which checks for geometry that does not match the
+ * manufacturing grid pitch.  This checks for the case of geometry that
+ * is finer than the underlying database grid, which can only happen for
+ * two non-Manhattan shapes interacting between two different cells, and
+ * can only be caught during the process of flattening the cell contents.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Creates a DRC error or prints the DRC error message, depending on
+ *	the value of drcSubFunc.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+DRCOffGridError(rect)
+    Rect      *rect;            /* Area of error */
+{
+    if (drcSubFunc == NULL) return;
+    (*drcSubFunc)(DRCErrorDef, rect, &drcOffGridCookie, drcSubClientData);
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -552,7 +697,7 @@ DRCInteractionCheck(def, area, erasebox, func, cdarg)
     int oldTiles, count, x, y, errorSaveType;
     Rect intArea, square, cliparea, subArea;
     PaintResultType (*savedPaintTable)[NT][NT];
-    void (*savedPaintPlane)();
+    int (*savedPaintPlane)();
     struct drcClientData arg;
     SearchContext scx;
 
@@ -585,6 +730,16 @@ DRCInteractionCheck(def, area, erasebox, func, cdarg)
             cliparea = square;
 	    GeoClip(&cliparea, area);
 
+	    /* Prepare for subcell search */
+	    DRCDummyUse->cu_def = def;
+	    scx.scx_use = DRCDummyUse;
+	    scx.scx_trans = GeoIdentityTransform;
+	    arg.dCD_celldef = def;
+	    arg.dCD_errors = &count;
+	    arg.dCD_cptr = &drcInSubCookie;
+	    arg.dCD_function = func;
+	    arg.dCD_clientData = cdarg;
+
 	    /* Find all the interactions in the square, and clip to the error
 	     * area we're interested in. */
 
@@ -600,6 +755,13 @@ DRCInteractionCheck(def, area, erasebox, func, cdarg)
 		errorSaveType = DRCErrorType;
 		DRCErrorType = TT_ERROR_P;	// Basic check is always ERROR_P
                 DRCBasicCheck(def, &intArea, &subArea, func, cdarg);
+
+		/* Copy errors up from all non-interacting children	*/
+		scx.scx_area = subArea;
+	    	arg.dCD_clip = &subArea;
+		DBCellSrArea(&scx, drcSubCopyFunc, &arg);
+		DBCellSrArea(&scx, drcArrayFunc, &arg);
+
 		DRCErrorType = errorSaveType;
 		continue;
 	    }
@@ -625,6 +787,7 @@ DRCInteractionCheck(def, area, erasebox, func, cdarg)
 		eraseClip = *erasebox;
 		GeoClip(&eraseClip, &cliparea);
 		subArea = eraseClip;
+	    	arg.dCD_clip = &subArea;
 
 		/* check above */
 		if (intArea.r_ytop < eraseClip.r_ytop)
@@ -632,6 +795,10 @@ DRCInteractionCheck(def, area, erasebox, func, cdarg)
 		    subArea.r_ybot = intArea.r_ytop;
 		    GEO_EXPAND(&subArea, DRCTechHalo, &eraseHalo);
                     DRCBasicCheck(def, &eraseHalo, &subArea, func, cdarg);
+		    /* Copy errors up from all non-interacting children	*/
+		    scx.scx_area = subArea;
+		    DBCellSrArea(&scx, drcSubCopyFunc, &arg);
+		    DBCellSrArea(&scx, drcArrayFunc, &arg);
 		}
 		/* check below */
 		if (intArea.r_ybot > eraseClip.r_ybot)
@@ -640,6 +807,10 @@ DRCInteractionCheck(def, area, erasebox, func, cdarg)
 		    subArea.r_ytop = intArea.r_ybot;
 		    GEO_EXPAND(&subArea, DRCTechHalo, &eraseHalo);
                     DRCBasicCheck(def, &eraseHalo, &subArea, func, cdarg);
+		    /* Copy errors up from all non-interacting children	*/
+		    scx.scx_area = subArea;
+		    DBCellSrArea(&scx, drcSubCopyFunc, &arg);
+		    DBCellSrArea(&scx, drcArrayFunc, &arg);
 		}
 		subArea.r_ytop = intArea.r_ytop;
 		subArea.r_ybot = intArea.r_ybot;
@@ -650,6 +821,10 @@ DRCInteractionCheck(def, area, erasebox, func, cdarg)
 		    subArea.r_xbot = intArea.r_xtop;
 		    GEO_EXPAND(&subArea, DRCTechHalo, &eraseHalo);
                     DRCBasicCheck(def, &eraseHalo, &subArea, func, cdarg);
+		    /* Copy errors up from all non-interacting children	*/
+		    scx.scx_area = subArea;
+		    DBCellSrArea(&scx, drcSubCopyFunc, &arg);
+		    DBCellSrArea(&scx, drcArrayFunc, &arg);
 		}
 		/* check left */
 		if (intArea.r_xbot > eraseClip.r_xbot)
@@ -658,6 +833,10 @@ DRCInteractionCheck(def, area, erasebox, func, cdarg)
 		    subArea.r_xbot = eraseClip.r_xbot;
 		    GEO_EXPAND(&subArea, DRCTechHalo, &eraseHalo);
                     DRCBasicCheck(def, &eraseHalo, &subArea, func, cdarg);
+		    /* Copy errors up from all non-interacting children	*/
+		    scx.scx_area = subArea;
+		    DBCellSrArea(&scx, drcSubCopyFunc, &arg);
+		    DBCellSrArea(&scx, drcArrayFunc, &arg);
 		}
 		DRCErrorType = errorSaveType;
 	    }
@@ -672,15 +851,13 @@ DRCInteractionCheck(def, area, erasebox, func, cdarg)
 
 	    DRCstatInteractions += 1;
 	    GEO_EXPAND(&intArea, DRCTechHalo, &scx.scx_area);
-	    DRCDummyUse->cu_def = def;
-	    scx.scx_use = DRCDummyUse;
-	    scx.scx_trans = GeoIdentityTransform;
 	    DBCellClearDef(DRCdef);
 
 	    savedPaintTable = DBNewPaintTable(DRCCurStyle->DRCPaintTable);
 	    savedPaintPlane = DBNewPaintPlane(DBPaintPlaneMark);
 
-	    (void) DBCellCopyAllPaint(&scx, &DBAllButSpaceBits, 0, DRCuse);
+	    (void) DBCellCheckCopyAllPaint(&scx, &DBAllButSpaceBits, 0,
+			DRCuse, func);
 
 	    (void) DBNewPaintTable(savedPaintTable);
 	    (void) DBNewPaintPlane(savedPaintPlane);
@@ -696,15 +873,10 @@ DRCInteractionCheck(def, area, erasebox, func, cdarg)
 
 	    /* Check for illegal partial overlaps. */
 
-	    scx.scx_use = DRCDummyUse;
 	    scx.scx_area = intArea;
-	    scx.scx_trans = GeoIdentityTransform;
-	    arg.dCD_celldef = DRCdef;
 	    arg.dCD_clip = &intArea;
-	    arg.dCD_errors = &count;
+	    arg.dCD_celldef = DRCdef;
 	    arg.dCD_cptr = &drcSubcellCookie;
-	    arg.dCD_function = func;
-	    arg.dCD_clientData = cdarg;
 	    (void) DBTreeSrUniqueTiles(&scx, &DRCCurStyle->DRCExactOverlapTypes,
 			0, drcExactOverlapTile, (ClientData) &arg);
 	}
@@ -725,7 +897,7 @@ DRCFlatCheck(use, area)
     SearchContext scx;
     void drcIncCount();
     PaintResultType (*savedPaintTable)[NT][NT];
-    void (*savedPaintPlane)();
+    int (*savedPaintPlane)();
     int drcFlatCount = 0;
 
     UndoDisable();
