@@ -4,16 +4,16 @@
  * Procedures to flatten the hierarchical description built
  * by efReadDef().
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -62,7 +62,12 @@ int efAddNodes(HierContext *, bool);
 int efAddConns(HierContext *, bool);
 int efAddOneConn(HierContext *, char *, char *, Connection *, bool);
 
-
+/* Flags passed to efFlatNode() */
+
+#define FLATNODE_STDCELL    0x01
+#define FLATNODE_DOWARN	    0x02
+
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -127,7 +132,10 @@ EFFlatBuild(name, flags)
 	if (flags & EF_NOFLATSUBCKT)
 	    efFlatNodesStdCell(&efFlatContext);
 	else
-	    efFlatNodes(&efFlatContext, FALSE, TRUE);
+	{
+	    int flags = FLATNODE_DOWARN;    /* No FLATNODE_STDCELL flag */
+	    efFlatNodes(&efFlatContext, (ClientData)flags);
+	}
 	efFlatKills(&efFlatContext);
 	if (!(flags & EF_NONAMEMERGE))
 	    efFlatGlob();
@@ -166,6 +174,7 @@ EFFlatBuildOneLevel(def, flags)
     Use *use;
     int efFlatNodesDeviceless();	/* Forward declaration */
     int efFlatCapsDeviceless();		/* Forward declaration */
+    int flatnodeflags;
 
     efFlatRootDef = def;
 
@@ -195,7 +204,8 @@ EFFlatBuildOneLevel(def, flags)
     efFlatRootUse.use_def = efFlatRootDef;
 
     /* Record all nodes down the hierarchy from here */
-    efFlatNodes(&efFlatContext, (ClientData)TRUE, (ClientData)FALSE);
+    flatnodeflags = 0;    /* No FLATNODE_DOWARN */
+    efFlatNodes(&efFlatContext, (ClientData)flatnodeflags);
 
     /* Expand all subcells that contain connectivity information but	*/
     /* no active devices (including those in subcells).			*/
@@ -209,10 +219,6 @@ EFFlatBuildOneLevel(def, flags)
     if ((usecount == 0) && (HashGetNumEntries(&efFlatRootUse.use_def->def_devs) == 0))
 	efFlatRootUse.use_def->def_flags |= DEF_NODEVICES;
 
-    /* Record all local nodes */
-    efAddNodes(&efFlatContext, FALSE);
-    efAddConns(&efFlatContext, TRUE);
-
     efFlatKills(&efFlatContext);
     if (!(flags & EF_NONAMEMERGE))
 	efFlatGlob();
@@ -223,7 +229,7 @@ EFFlatBuildOneLevel(def, flags)
 
     return &efFlatContext;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -258,7 +264,7 @@ EFFlatDone()
     HashKill(&efHNUseHashTable);
     return;
 }
-
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -287,15 +293,24 @@ EFFlatDone()
  *	Adds node names to the table of flattened node names efNodeHashTable.
  *	May merge nodes from the list efNodeList as per the connection
  *	list hc->hc_use->use_def->def_conns.
+ * 
+ * Note:
+ *	stdcell = TRUE is only used when writing DEF files.
  *
  * ----------------------------------------------------------------------------
  */
 
 int
-efFlatNodes(hc, stdcell, doWarn)
+efFlatNodes(hc, clientData)
     HierContext *hc;
+    ClientData clientData;
 {
-    (void) efHierSrUses(hc, efFlatNodes);
+    int flags = (int)clientData;
+
+    bool stdcell = (flags & FLATNODE_STDCELL) ? TRUE : FALSE;
+    bool doWarn = (flags & FLATNODE_DOWARN) ? TRUE : FALSE;
+
+    (void) efHierSrUses(hc, efFlatNodes, clientData);
 
     /* Add all our own nodes to the table */
     efAddNodes(hc, stdcell);
@@ -461,11 +476,12 @@ efAddNodes(hc, stdcell)
 
 	// If called with "hierarchy on", all local node caps and adjustments
 	// have been output and should be ignored.
-	
+
 	newnode->efnode_cap = (!stdcell) ? node->efnode_cap : (EFCapValue)0.0;
 	newnode->efnode_client = (ClientData) NULL;
 	newnode->efnode_flags = node->efnode_flags;
 	newnode->efnode_type = node->efnode_type;
+	newnode->efnode_num = 1;
 	if (!stdcell)
 	    bcopy((char *) node->efnode_pa, (char *) newnode->efnode_pa,
 			efNumResistClasses * sizeof (EFPerimArea));
@@ -482,15 +498,15 @@ efAddNodes(hc, stdcell)
 	newnode->efnode_loc.r_ybot = (int)((float)(newnode->efnode_loc.r_ybot) * scale);
 	newnode->efnode_loc.r_ytop = (int)((float)(newnode->efnode_loc.r_ytop) * scale);
 
+	/* Add each name for this node to the hash table */
+	newnode->efnode_name = (EFNodeName *) NULL;
+
 	/* Prepend to global node list */
 	newnode->efnode_next = efNodeList.efnode_next;
 	newnode->efnode_prev = (EFNodeHdr *) &efNodeList;
 	efNodeList.efnode_next->efnhdr_prev = (EFNodeHdr *) newnode;
 	efNodeList.efnode_next = (EFNodeHdr *) newnode;
 
-	/* Add each name for this node to the hash table */
-	newnode->efnode_name = (EFNodeName *) NULL;
-
 	for (nn = node->efnode_name; nn; nn = nn->efnn_next)
 	{
 	    /*
@@ -514,7 +530,7 @@ efAddNodes(hc, stdcell)
 		if (hierName != nn->efnn_hier)
 		    EFHNFree(hierName, hc->hc_hierName, HN_CONCAT);
 		if (oldname->efnn_node != newnode)
-		    efNodeMerge(oldname->efnn_node, newnode);
+		    efNodeMerge(&oldname->efnn_node, &newnode);
 		newnode = oldname->efnn_node;
 		continue;
 	    }
@@ -540,6 +556,7 @@ efAddNodes(hc, stdcell)
 		newnode->efnode_name = newname;
 	    }
 	}
+
     }
     return 0;
 }
@@ -585,7 +602,7 @@ efAddConns(hc, doWarn)
 
     return (0);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -638,12 +655,12 @@ efAddOneConn(hc, name1, name2, conn, doWarn)
 	    return 0;
 	newnode = ((EFNodeName *) HashGetValue(he2))->efnn_node;
 	if (node != newnode)
-	    efNodeMerge(node, newnode);
+	    efNodeMerge(&node, &newnode);
     }
 
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -744,7 +761,7 @@ efFlatGlob()
 	    {
 		efFlatGlobError(nameGlob, nameFlat);
 	    }
-	    efNodeMerge(nodeFlat, nodeGlob);
+	    efNodeMerge(&nodeFlat, &nodeGlob);
 	    nameGlob->efnn_node = nodeFlat;
 	}
     }
@@ -854,7 +871,7 @@ efFlatGlobHash(hierName)
 {
     return hierName->hn_hash;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -898,7 +915,7 @@ efFlatKills(hc)
 
     return (0);
 }
-
+
 
 /*----
  * WIP
@@ -990,7 +1007,7 @@ efFlatCaps(hc)
 
     return (0);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1025,7 +1042,7 @@ efFlatSingleCap(hc, name1, name2, conn)
     static char msg0[] = "cap(1)";
     static char msg1[] = "cap(2)";
     char *msg;
-    
+
     /* Connections that are below threshold (ext2spice hierarchy only)	*/
     /* will be missing.  Do not generate errors for these.		*/
 
@@ -1063,7 +1080,7 @@ efFlatSingleCap(hc, name1, name2, conn)
 
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1150,14 +1167,14 @@ HashEntry *he;
  * CapHashSetValue()
  *       if the pointer is null, allocate a EFCapValue and point to it.
  *        Then copy in the new value.
- * 
+ *
  * need to pass doubles regardless of what CapValue is because of
  * argument promotion in ANSI C
  *
  */
 void
 CapHashSetValue(he, c)
-HashEntry *he; 
+HashEntry *he;
 double c;
 {
 	EFCapValue *capp = (EFCapValue *)HashGetValue(he);
