@@ -340,6 +340,7 @@ extHardSetLabel(scx, reg, arg)
     }
     GeoTransRect(&scx->scx_trans, &r, &newlab->lab_rect);
     newlab->lab_type = oldlab->lab_type;
+    newlab->lab_flags = oldlab->lab_flags;
     text = oldlab->lab_text;
 
     /* Don't care, really, which orientation the label has */
@@ -405,30 +406,15 @@ extHardGenerateLabel(scx, reg, arg)
     Rect r;
     Point p;
 
-    // Modification 9/9/2014 by Tim:
-    // Convert the treg_ll value up to top-level coordinates.
-    // Otherwise you end up with a node that is apparently in
-    // "canonical coordinates", but if you try to find the
-    // location of the node using the name, you'll end up in
-    // a random place.  It also allows the low-probability
-    // but possible conflict between this node and another with
-    // the same name in the parent cell.
-    //
-    // Reverted 10/30/2014:  Apparently this causes worse
-    // problems.
-    //
-    // GeoTransPoint(&scx->scx_trans, &reg->treg_ll, &r.r_ll);
-    // extMakeNodeNumPrint(gen, reg->treg_pnum, r.r_ll);
-
-    extMakeNodeNumPrint(gen, reg->treg_pnum, reg->treg_ll);
+    extMakeNodeNumPrint(gen, (LabRegion *)reg);
 
     prefixlen = tpath->tp_next - tpath->tp_first;
     len = strlen(gen) + prefixlen;
     newlab = (Label *) mallocMagic((unsigned) (sizeof (Label) + len - 3));
     r.r_ll = reg->treg_tile->ti_ll;
-    GEOCLIP(&r,&scx->scx_area);
     r.r_ur.p_x = r.r_ll.p_x+1;
     r.r_ur.p_y = r.r_ll.p_y+1;
+    GEOCLIP(&r,&scx->scx_area);
     GeoTransRect(&scx->scx_trans, &r, &newlab->lab_rect);
     newlab->lab_type = TiGetType(reg->treg_tile);
 
